/**********************************************************
 **         Class for Config-file using STL only.        **
 **                 anselm.su [2013-01-14]               **
 **********************************************************/
 
#include "conf.h"
#include <fstream>
#include <iostream>

#ifdef WIN32
#include <windows.h>    //GetModuleFileNameA
#include <shlwapi.h>    //PathFindFileNameA
#else
#define MAX_PATH 256
#include <libgen.h>     //basename
#include <dlfcn.h>      //dladdr
#include <string.h>     //strdup
#endif


using namespace std;

Conf::Conf(const char *name, const char *rem)
  : d_size(0)
{
  d_name = new char[MAX_PATH];
  memset(d_name, 0, MAX_PATH);
  if(rem)
    d_rem = strdup(rem);
  else
    d_rem = strdup("");
  guess(name);
}

Conf::~Conf() 
{
  delete [] d_name;
  delete [] d_rem;
}



bool Conf::module_name(char* a)
{
  bool cool;
#ifdef WIN32
  cool = GetModuleFileNameA(0, a, MAX_PATH) > 0;
#else  
  Dl_info info;
  cool = dladdr((void*)module_name, &info);
  if(cool) {
    strcpy(a, info.dli_fname);
  }
#endif
  return cool;
}


bool Conf::open(const char* name)
{
  ifstream f(name, ios::binary);
  d_exist = f.good();
  if(!d_exist) {    
    f.close();
    return false;
  }
  
  
  // Templates for sscanf
  const char *T1 = "[%%[^]%s]]";
  const char *T2 = "%%[^=%s]=%%[^%s]";
  char t1[50] = {0};
  char t2[50] = {0};
  sprintf(t1, T1, d_rem);
  sprintf(t2, T2, d_rem, d_rem);
  
  String s, section_name="";
  while( !f.eof() ) {
    getline(f, s, '\n');
  
    String s1(""), s2(""), s3("");
    //s.sscanf("[%[^]#]]", &s1);
    //s.sscanf("%[^=#]=%[^#]", &s2, &s3);
    s.sscanf(t1, &s1);
    s.sscanf(t2, &s2, &s3);
    
    if( !s1.trim().empty() ) {
      section_name = s1;
      //cout << "[" << section_name << "]" << endl;
    }
    
    else if( !section_name.empty() && !s2.trim().empty()) {
      //cout << s2 << endl;
      //(*this)[section_name][ s2 ] = "";
      (*this)[section_name][ s2 ] = s3.trim();      
    }//else if
  }//while
  
  f.close();
  
  return true;
}

bool Conf::guess(const char* name)
{
#ifdef WIN32
  module_name(d_name);
  strlwr(d_name);
  if(name) {
    char *bname = PathFindFileNameA(name);
    char *last_slash = strrchr(d_name, '\\');
    *(++last_slash) = 0;
    strcat(d_name, bname);
  }
  else {
    char *e = strstr(d_name, ".exe");
    strcpy(e, ".conf");
  }
#else
  if(name) {
    char *bname = basename((char*)name);
    if(strcmp(bname,name)!=0) {
      strcpy(d_name, name);
      return open(d_name);                           // find strongly
    }
    
    if( module_name(d_name) ) {
      char *last_slash = strrchr(d_name, '/');
      *(++last_slash) = 0;
      strcat(d_name, bname);
      if(open(d_name))                               // find in local
        return true;
      else
        sprintf(d_name, "/usr/local/etc/%s", bname); // find in etc
    }
    else {
      sprintf(d_name, "/usr/local/etc/%s", bname);   // find in etc
    }
  }
  
  else if( getenv("conffile")!=NULL )                    // if envirenment
    strcpy(d_name, getenv("conffile") );
  else {
    if(module_name(d_name)) {
      
      strcat(d_name, ".conf");
      if(open(d_name))                               // find in local
        return true;
      else {
        char *bname = basename(d_name);
        sprintf(d_name, "/usr/local/etc/%s", bname); // find in etc
      }
      
    }
    else
      return false;
  }
#endif

  return open(d_name);
}


const char* Conf::readString(const char* section, const char* key, const char* defaults)
{
  return (*this)[section][key].asString(defaults);
}

int Conf::readInteger(const char* section, const char* key, const int defaults)
{
  return (*this)[section][key].asInteger(defaults);
}

double Conf::readDouble(const char* section, const char* key, const double defaults)
{
  return (*this)[section][key].asDouble(defaults);
}

bool Conf::readBoolean(const char* section, const char* key, const bool defaults)
{
  return (*this)[section][key].asBoolean(defaults);
}

Section& Conf::readSection(const char* section)
{
  return (*this)[section];
}


/**************************************************************************************************
                                            Section
**************************************************************************************************/
const char* Section::join(const char* term)
{
  string s;
  Section::iterator I = this->begin();
  for(; I!=this->end(); I++) {
    s += I->first;
    if(!I->second.empty()) s += "=" + I->second;
    s += term;
  }
  return s.c_str();
}
